Imports System.IO
Imports System.Text

Public Class Rsspecan_Scan_Measurement_Example

    Private Sub Rsspecan_Scan_Measurement_Example_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Detector.SelectedIndex = 0
    End Sub

    Private Sub ExitButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitButton.Click
        Close()
    End Sub

    Private Sub MeasureAndSaveButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MeasureAndSaveButton.Click

        Dim m_Instrument As rsspecan
        Dim instr_modl As String
        Dim trace1(10000) As Double
        Dim trace2(10000) As Double
        Dim trace3(10000) As Double
        Dim trace4(10000) As Double
        Dim trace5(10000) As Double
        Dim trace6(10000) As Double
        Dim traceData(20000) As Double
        Dim statusReg, traceStatus, noofPoints, pointsToPrint, i As Integer
        Dim saveFileSW As StreamWriter = Nothing
        Dim statusData As New StringBuilder(10000)
        Dim errorCode As Integer
        Dim errorMessage As New StringBuilder(1024)

        m_Instrument = Nothing
        statusReg = traceStatus = noofPoints = pointsToPrint = 0
        errorCode = 0

        Try
            If SaveMeasuredDataDialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
                UseWaitCursor = True
                System.Windows.Forms.Cursor.Current = Cursors.WaitCursor

                'initialize driver
                m_Instrument = New rsspecan(ResourceDescriptor.Text, IDQuery.Checked, ResetDevice.Checked)

                'check for EMI test receiver
                instr_modl = m_Instrument.GetString(rsspecanProperties.RsAttrInstrumentModel, "")
                If (instr_modl.IndexOf("ESL") = -1) Then
                    UseWaitCursor = False
                    System.Windows.Forms.Cursor.Current = Cursors.Default
                    MessageBox.Show("Instrument does not support EMI scan measurements", "Error")
                    Return
                End If

                'switch to receiver mode
                m_Instrument.ReceiverMode()

                'switch device display on/off
                m_Instrument.ConfigureDisplayUpdate(DisplayEnable.Checked)

                'set one scan at 0.1 milliseconds
                m_Instrument.ConfigureScanTableMeasTime(1, 0.0001)

                'set scan step mode
                m_Instrument.ConfigureReceiverStepMode(rsspecanConstants.SweepSpacLin)

                'switch to single sweep
                m_Instrument.ConfigureAcquisition(2, False, 1)

                'for security reasons
                m_Instrument.ConfigureReceiverTracePoints(10000)

                'Set start and stop frequency and the stepsize
                m_Instrument.ConfigureScanTableStartStop(1, Convert.ToDouble(StartFreq.Value), Convert.ToDouble(StopFreq.Value), Convert.ToDouble(FreqStep.Value))

                'Send trace datas before trace is finished, too
                m_Instrument.ConfigureReceiverTraceFeedControl(rsspecanConstants.TracFeedAlways)

                m_Instrument.ConfigureFinalReceiverDetector(1, 1, rsspecanConstants.DetectorTypeMinPeak)

                'Select detector
                If (Detector.SelectedIndex = 0) Then 'Average
                    m_Instrument.ConfigureReceiverDetector(1, 1, True, False, False, False, False, False, False)
                ElseIf (Detector.SelectedIndex = 1) Then  'Max Peak
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, True, False, False, False, False, False)
                ElseIf (Detector.SelectedIndex = 2) Then  'Min Peak
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, False, True, False, False, False, False)
                ElseIf (Detector.SelectedIndex = 3) Then  'RMS
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, False, False, True, False, False, False)
                ElseIf (Detector.SelectedIndex = 4) Then  'QPeak
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, False, False, False, True, False, False)
                ElseIf (Detector.SelectedIndex = 5) Then  'CAverage
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, False, False, False, False, False, True)
                ElseIf (Detector.SelectedIndex = 6) Then  'CRMS
                    m_Instrument.ConfigureReceiverDetector(1, 1, False, False, False, False, False, True, False)
                End If

                m_Instrument.RunReceiverMeasurement()

                While ((traceStatus And 4096) = 0)

                    'Read status register
                    m_Instrument.getStatusRegister(1, statusReg)

                    If ((statusReg And 512) <> 0) Then

                        'Read Trace
                        m_Instrument.ReadScanMeasurement(10000, traceStatus, noofPoints, statusData, trace1, trace2, trace3, trace4, trace5, trace6)

                        trace1.CopyTo(traceData, pointsToPrint)

                        pointsToPrint += noofPoints

                    Else
                        System.Threading.Thread.Sleep(50)
                    End If
                End While

                'open save file
                saveFileSW = New StreamWriter(SaveMeasuredDataDialog.FileName, False, Encoding.Default)

                'write data to file
                For i = 0 To pointsToPrint
                    saveFileSW.Write(traceData(i).ToString() + ";")
                Next i

            End If
        Catch ex As Exception
            Dim message As String
            If Not (m_Instrument Is Nothing) Then
                m_Instrument.error_query(errorCode, errorMessage)
            End If
            message = "Instrument Status Error: "
            message += errorMessage.ToString()
            message += ex.Message
            MessageBox.Show(message)

        Finally
            'close file if opened
            If Not (saveFileSW Is Nothing) Then
                saveFileSW.Close()
            End If

            UseWaitCursor = False
            System.Windows.Forms.Cursor.Current = Cursors.Default
            If Not (m_Instrument Is Nothing) Then
                m_Instrument.Dispose()
            End If
        End Try
    End Sub
End Class
